@@ -4,12 +4,14 @@ from arrow import Arrow
 # Local imports
 from . import validators
 
-DUST = 546
+DUST = 1332
 PUBLIC_KEY_BYTES_LENGTH = 33
 SCRIPT_INT_MAX_WHEN_JAVASCRIPT = int(2 ** 53)
+SCRIPT_INT_MAX = int(2 ** 63 - 1)
 SATS_PER_BCH = int(1e8)
-MAX_REASONABLE_SATS = int(10e6 * SATS_PER_BCH)  # 10 million BCH
+MAX_REASONABLE_SATS = int(100_000 * SATS_PER_BCH)  # 100k BCH
 MIN_NLOCKTIME_TIMESTAMP = 500_000_000
+MIN_REASONABLE_DIVISION_STEPS = 500
 
 
 class PublicKey(str):
