@@ -59,12 +59,12 @@ class StrictPricepoints:
         validators.equal(unique_pricepoints, sorted_pricepoints)
 
     @staticmethod
-    def _sort_pricepoints(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint]:
+    def _sort_pricepoints(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint, ...]:
         return tuple(sorted(pricepoints, key=lambda p: p.timestamp))
 
     @staticmethod
-    def _make_pricepoints_unique(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint]:
-        lookup = {}
+    def _make_pricepoints_unique(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint, ...]:
+        lookup: dict[ScriptTimestamp, Pricepoint] = {}
         for p in pricepoints:
             existing = lookup.get(p.timestamp)
             if existing is None:
@@ -239,6 +239,13 @@ class EthEM5(OracleUnit):
     public_key = PublicKey('038ab22e37cf020f6bbef40111ddc51083a936f0821de56ac01f799cf15b87904d')
 
 
+class EurEM2(OracleUnit):
+    name_of_oracleUnits = 'EUR(e-2)'
+    name_of_standardUnits = 'EUR'
+    oracleUnits_per_standardUnit = 100
+    public_key = PublicKey('02bb9b3324df889a66a57bc890b3452b84a2a74ba753f8842b06bba03e0fa0dfc5')
+
+
 class InrEM0(OracleUnit):
     name_of_oracleUnits = 'INR'
     name_of_standardUnits = 'INR'
@@ -333,7 +340,7 @@ class UsdEM2Testing(OracleUnit):
 
 
 known_unit_classes = [
-    BtcEM6, CnyEM2, DogeEM1, EthEM5, InrEM0, UsdEM2, XagEM3, XauEM5,
+    BtcEM6, CnyEM2, DogeEM1, EthEM5, EurEM2, InrEM0, UsdEM2, XagEM3, XauEM5,
     BtcEM6Beta, CnyEM2Beta, DogeEM1Beta, EthEM5Beta, InrEM0Beta, UsdEM2Beta, XagEM3Beta, XauEM5Beta,
     UsdEM2Testing,
 ]
