@@ -13,15 +13,20 @@ from ..bch_primitives import (
     UtxoSats,
 )
 from ..contract import (
+    ContractFromInputSatsAndLeverages,
+    ContractFromNominalHedgeAndLeverages,
     ContractFunding,
     ContractProposal,
     ContractRedemption,
+    RedemptionType,
+    UnredeemableError,
+)
+from ..contract_primitives import (
     LongLeverage,
     NominalOracleUnitsXSatsPerBch,
-    RedemptionType,
+    Role,
     ShortLeverage,
     Side,
-    UnredeemableError,
 )
 from ..fee import FeeAgreement
 from ..oracle import (
@@ -29,7 +34,6 @@ from ..oracle import (
     ScriptPriceInOracleUnitsPerBch,
     UsdEM2Beta,
 )
-from ..role import Role
 
 
 ###################
@@ -37,6 +41,7 @@ from ..role import Role
 ###################
 # Generic intent values, not always perfectly reproducible from the contract
 START_TIMESTAMP = ScriptTimestamp(Arrow(year=2021, month=10, day=21).int_timestamp)
+LIQUIDATION_TIMESTAMP = ScriptTimestamp(Arrow(year=2021, month=10, day=22).int_timestamp)  # 1 days later
 MATURITY_TIMESTAMP = ScriptTimestamp(Arrow(year=2021, month=10, day=23).int_timestamp)  # 2 days later
 NOMINAL_VALUE_USDEM2 = UsdEM2Beta(100_00.0)  # $100.00
 LONG_LEVERAGE = LongLeverage(4.5)
@@ -68,8 +73,8 @@ LOW_LIQUIDATION_PRICE = ScriptPriceInOracleUnitsPerBch(155_56)
 
 # High liquidation Price is the price where all of short's input is consumed to cover long's agreed payout
 # When short is exactly a 1x short, there is no meaningful liquidation price (infinity) but the stored liquidation price is still
-# used to trigger early settlement. By definition, we set it to the max possible price in the 4 byte oracle price message scheme.
-HIGH_LIQUIDATION_PRICE_AS_HEDGE = ScriptPriceInOracleUnitsPerBch.largest_allowed()
+# used to trigger early settlement. By definition, we set it to 10x start price
+HIGH_LIQUIDATION_PRICE_AS_HEDGE = ScriptPriceInOracleUnitsPerBch(10 * START_PRICE_USDEM2_PER_BCH)
 # In the case of a leveraged short, it is calculated as round(START_PRICE_USDEM2_PER_BCH * (1 + 1 / (SHORT_LEVERAGE_NOT_1 - 1)))
 # 36667 cents/BCH ($366.67 / BCH)
 HIGH_LIQUIDATION_PRICE_AS_LEVERAGED_SHORT = ScriptPriceInOracleUnitsPerBch(366_67)
@@ -93,8 +98,8 @@ SHORT_INPUT_SATS_AS_LEVERAGED_SHORT = UtxoSats(22_727_521)
 # Similar to cost at high liquidation, there is also the cost at low liquidation.
 # In the contract, this is not stored anywhere, but it is a useful intermediary for confirming tests.
 # It is the amount needed to cover the nominal unit value at low liquidation price
-# Calculated as ceil((NOMINAL_UNITS_X_SATS_PER_BCH) / DERIVED_LIQUIDATION_PRICE_UNITS_PER_BCH)
-COST_SATS_FOR_NOMINAL_VALUE_AT_LOW_LIQUIDATION = UtxoSats(64_283_878)
+# Calculated as (NOMINAL_UNITS_X_SATS_PER_BCH // DERIVED_LIQUIDATION_PRICE_UNITS_PER_BCH)
+COST_SATS_FOR_NOMINAL_VALUE_AT_LOW_LIQUIDATION = UtxoSats(64_283_877)
 
 # The total input sats needed to cover all positions is the sum of the short and long inputs.
 # However, from base contract values, it is the difference between the cost at the two liquidation points
@@ -102,19 +107,19 @@ COST_SATS_FOR_NOMINAL_VALUE_AT_LOW_LIQUIDATION = UtxoSats(64_283_878)
 TOTAL_INPUT_SATS_AS_HEDGE = COST_SATS_FOR_NOMINAL_VALUE_AT_LOW_LIQUIDATION - 0
 # For a leveraged short, it has to cover the cost between low liquidation and high liquidation (which also might be zero)
 # Calculated as COST_SATS_FOR_NOMINAL_VALUE_AT_LOW_LIQUIDATION - COST_SATS_FOR_NOMINAL_VALUE_AT_HIGH_LIQUIDATION
-TOTAL_INPUT_SATS_AS_LEVERAGED_SHORT = UtxoSats(37_011_399)
+TOTAL_INPUT_SATS_AS_LEVERAGED_SHORT = UtxoSats(37_011_398)
 
 # The bch amount to provide volatility protection as specified by the multiplier
 # This is the amount to cover the difference between payout bch at start and payout bch at long's liquidation price
 # Calculated as DERIVED_TOTAL_INPUT_SATS - SHORT_INPUT_SATS_AS_HEDGE
-LONG_INPUT_SATS = UtxoSats(14_283_878)
+LONG_INPUT_SATS = UtxoSats(14_283_877)
 
 # Converting the input amounts into ORACLE units at start
 # for pure hedge, it's NOMINAL_UNITS_X_SATS_PER_BCH / SATS_PER_BCH
-SHORT_INPUT_ORACLE_UNITS_AS_HEDGE = UsdEM2Beta(100_00.0)
+SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START = UsdEM2Beta(100_00.0)
 # for long, it's LONG_INPUT_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH
-LONG_INPUT_ORACLE_UNITS = UsdEM2Beta(2856.7756)
-TOTAL_INPUT_ORACLE_UNITS_AS_HEDGE = UsdEM2Beta(SHORT_INPUT_ORACLE_UNITS_AS_HEDGE + LONG_INPUT_ORACLE_UNITS)
+LONG_INPUT_ORACLE_UNITS_AT_START = UsdEM2Beta(2856.7754)
+TOTAL_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START = UsdEM2Beta(SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START + LONG_INPUT_ORACLE_UNITS_AT_START)
 
 
 def standard_contract_proposal(
@@ -126,15 +131,16 @@ def standard_contract_proposal(
     maker_side: Side = Side.SHORT,
     short_leverage: ShortLeverage = SHORT_LEVERAGE_1_AS_HEDGE,
 ) -> ContractProposal:
-    return ContractProposal.new_from_intent(
+    intent = ContractFromNominalHedgeAndLeverages(
         start_timestamp=start_timestamp,
         maturity_timestamp=maturity_timestamp,
         nominal_oracleUnits=nominal_oracleUnits,
+        short_leverage=short_leverage,
         long_leverage=long_leverage,
         start_price_oracleUnits_per_bch=start_price_oracleUnits_per_bch,
         maker_side=maker_side,
-        short_leverage=short_leverage,
     )
+    return intent.proposal
 
 
 class TestContractProposalDerivedValues(unittest.TestCase):
@@ -190,13 +196,13 @@ class TestContractProposalDerivedValues(unittest.TestCase):
     # Unit values
     ######################
     def test_total_input_oracleUnits(self):
-        self.assertEqual(standard_contract_proposal().total_input_oracleUnits, TOTAL_INPUT_ORACLE_UNITS_AS_HEDGE)
+        self.assertEqual(standard_contract_proposal().total_input_oracleUnits, TOTAL_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START)
 
     def test_short_input_oracleUnits(self):
-        self.assertEqual(standard_contract_proposal().short_input_oracleUnits, SHORT_INPUT_ORACLE_UNITS_AS_HEDGE)
+        self.assertEqual(standard_contract_proposal().short_input_oracleUnits, SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START)
 
     def test_long_input_oracleUnits(self):
-        self.assertEqual(standard_contract_proposal().long_input_oracleUnits, LONG_INPUT_ORACLE_UNITS)
+        self.assertEqual(standard_contract_proposal().long_input_oracleUnits, LONG_INPUT_ORACLE_UNITS_AT_START)
 
     ######################
     # Role versions of values
@@ -207,16 +213,16 @@ class TestContractProposalDerivedValues(unittest.TestCase):
         self.assertEqual(short_maker_contract.taker_side, Side.LONG)
         self.assertEqual(short_maker_contract.maker_input_sats, SHORT_INPUT_SATS_AS_HEDGE)
         self.assertEqual(short_maker_contract.taker_input_sats, LONG_INPUT_SATS)
-        self.assertEqual(short_maker_contract.maker_input_oracleUnits, SHORT_INPUT_ORACLE_UNITS_AS_HEDGE)
-        self.assertEqual(short_maker_contract.taker_input_oracleUnits, LONG_INPUT_ORACLE_UNITS)
+        self.assertEqual(short_maker_contract.maker_input_oracleUnits, SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START)
+        self.assertEqual(short_maker_contract.taker_input_oracleUnits, LONG_INPUT_ORACLE_UNITS_AT_START)
 
         long_maker_contract = standard_contract_proposal(maker_side=Side.LONG)
         self.assertEqual(long_maker_contract.maker_side, Side.LONG)
         self.assertEqual(long_maker_contract.taker_side, Side.SHORT)
         self.assertEqual(long_maker_contract.maker_input_sats, LONG_INPUT_SATS)
         self.assertEqual(long_maker_contract.taker_input_sats, SHORT_INPUT_SATS_AS_HEDGE)
-        self.assertEqual(long_maker_contract.maker_input_oracleUnits, LONG_INPUT_ORACLE_UNITS)
-        self.assertEqual(long_maker_contract.taker_input_oracleUnits, SHORT_INPUT_ORACLE_UNITS_AS_HEDGE)
+        self.assertEqual(long_maker_contract.maker_input_oracleUnits, LONG_INPUT_ORACLE_UNITS_AT_START)
+        self.assertEqual(long_maker_contract.taker_input_oracleUnits, SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START)
 
     ######################
     # Role versions of values
@@ -240,22 +246,24 @@ class TestContractProposalNeutralize(unittest.TestCase):
         # Use a base case and hand-constructed neutralizing contract to confirm actual result
 
         # Construct the base case
-        base_proposal = ContractProposal.new_from_intent(
+        base_intent = ContractFromNominalHedgeAndLeverages(
             start_timestamp=START_TIMESTAMP,
             maturity_timestamp=MATURITY_TIMESTAMP,
             nominal_oracleUnits=NOMINAL_VALUE_USDEM2,
+            short_leverage=ShortLeverage(1.0),
             long_leverage=LongLeverage(5.0),
             start_price_oracleUnits_per_bch=START_PRICE_USDEM2_PER_BCH,
             maker_side=Side.SHORT,
-            short_leverage=ShortLeverage(1.0),
-            address='',
         )
+        base_proposal = base_intent.proposal
 
         # Manually construct the neutralizing contract
         neutralizing_price = ScriptPriceInOracleUnitsPerBch(190_00)
         neutralizing_timestamp = ScriptTimestamp(round((START_TIMESTAMP + MATURITY_TIMESTAMP) / 2))
         expected_neutralizing_contract = ContractProposal(
             address='',
+            short_mutual_redeem_public_key='',
+            long_mutual_redeem_public_key='',
             start_timestamp=neutralizing_timestamp,
             maturity_timestamp=base_proposal.maturity_timestamp,
             nominal_oracleUnits_x_satsPerBch=base_proposal.nominal_oracleUnits_x_satsPerBch,
@@ -278,34 +286,33 @@ class TestContractProposalNeutralize(unittest.TestCase):
 class TestContractProposalValidation(unittest.TestCase):
     def test_ValueError_if_duration_too_short(self):
         with self.assertRaises(ValueError):
-            too_short_maturity_timestamp = ScriptTimestamp(START_TIMESTAMP + 60 - 1)
+            too_short_maturity_timestamp = ScriptTimestamp(START_TIMESTAMP + 0)
             standard_contract_proposal(maturity_timestamp=too_short_maturity_timestamp)
 
 
-# Create the standard set of fees
+# Create a set of fees to be used in funding and redemption
 FEE_100000_TAKER_TO_MAKER = FeeAgreement(name='fee type a', amount_sats=Sats(100000), receiving=Role.MAKER, paying=Role.TAKER)
 FEE_200000_TAKER_TO_MAKER = FeeAgreement(name='fee type b', amount_sats=Sats(200000), receiving=Role.MAKER, paying=Role.TAKER)
 FEE_400000_TAKER_TO_SETTLEMENT_SERVICE = FeeAgreement(name='fee type ss', amount_sats=Sats(400000), receiving=Role.SETTLEMENT_SERVICE, paying=Role.TAKER)
-FEES = (FEE_100000_TAKER_TO_MAKER, FEE_200000_TAKER_TO_MAKER, FEE_400000_TAKER_TO_SETTLEMENT_SERVICE)
 
+# Create the set of funding fees
+FUNDING_FEES = (FEE_100000_TAKER_TO_MAKER, FEE_200000_TAKER_TO_MAKER, FEE_400000_TAKER_TO_SETTLEMENT_SERVICE)
 
-# Group them for reference
-TOTAL_FEES_TO_MAKER_SATS = Sats(
+# Manually group them by side and unit
+TOTAL_FUNDING_FEES_TO_MAKER_SATS = Sats(
     + FEE_100000_TAKER_TO_MAKER.amount_sats
     + FEE_200000_TAKER_TO_MAKER.amount_sats
 )
-TOTAL_FEES_TO_MAKER_ORACLE_UNITS = UsdEM2Beta(TOTAL_FEES_TO_MAKER_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
-TOTAL_FEES_TO_TAKER_SATS = Sats(
+TOTAL_FUNDING_FEES_TO_TAKER_SATS = Sats(
     - FEE_100000_TAKER_TO_MAKER.amount_sats
     - FEE_200000_TAKER_TO_MAKER.amount_sats
     - FEE_400000_TAKER_TO_SETTLEMENT_SERVICE.amount_sats
 )
-TOTAL_FEES_TO_TAKER_ORACLE_UNITS = UsdEM2Beta(TOTAL_FEES_TO_TAKER_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
 
 
 def standard_contract_funding(proposal: ContractProposal | None = None) -> ContractFunding:
     proposal = proposal or standard_contract_proposal()
-    return proposal.fund(fee_agreements=FEES)
+    return proposal.fund(fee_agreements=FUNDING_FEES)
 
 
 class TestContractFundingDerivedValues(unittest.TestCase):
@@ -314,8 +321,8 @@ class TestContractFundingDerivedValues(unittest.TestCase):
         short_maker_contract = standard_contract_funding(standard_contract_proposal(maker_side=Side.SHORT))
         long_maker_contract = standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG))
 
-        expected_fees_sats_to_maker = TOTAL_FEES_TO_MAKER_SATS
-        expected_fees_sats_to_taker = TOTAL_FEES_TO_TAKER_SATS
+        expected_fees_sats_to_maker = TOTAL_FUNDING_FEES_TO_MAKER_SATS
+        expected_fees_sats_to_taker = TOTAL_FUNDING_FEES_TO_TAKER_SATS
 
         # Short Maker
         self.assertEqual(short_maker_contract.fee_sats_to_short, expected_fees_sats_to_maker)
@@ -339,8 +346,8 @@ class TestContractFundingDerivedValues(unittest.TestCase):
         long_maker_contract = standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG))
         oracle_class = short_maker_contract.base_proposal.oracle_unit_cls
 
-        expected_fee_oracleUnits_to_maker_at_funding = oracle_class(TOTAL_FEES_TO_MAKER_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
-        expected_fee_oracleUnits_to_taker_at_funding = oracle_class(TOTAL_FEES_TO_TAKER_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+        expected_fee_oracleUnits_to_maker_at_funding = oracle_class(TOTAL_FUNDING_FEES_TO_MAKER_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+        expected_fee_oracleUnits_to_taker_at_funding = oracle_class(TOTAL_FUNDING_FEES_TO_TAKER_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
 
         # Short Maker
         self.assertEqual(short_maker_contract.fee_oracleUnits_to_short, expected_fee_oracleUnits_to_maker_at_funding)
@@ -452,28 +459,43 @@ SHORT_PAYOUT_SATS_AS_HEDGE = UtxoSats(45_454_545)  # 0.45454545... Bch
 SHORT_PAYOUT_SATS_AS_LEVERAGED_SHORT = UtxoSats(18_182_066)
 
 # Calculated as SHORT_PAYOUT_SATS_AS_HEDGE * MATURITY_PRICE / SATS_PER_BCH
-SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE = UsdEM2Beta(9999.999900)  # 99.99999900000 USD
+SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE_AT_REDEMPTION = UsdEM2Beta(9999.999900)  # 99.99999900000 USD
 
 # Derived long payout sats and units
 # Calculated as max(dust, TOTAL_INPUT - HEDGE_PAYOUT)
-LONG_PAYOUT_SATS = UtxoSats(18_829_333)  # 0.18829... Bch
+LONG_PAYOUT_SATS = UtxoSats(18_829_332)  # 0.18829... Bch
 # Calculated as LONG_PAYOUT_SATS * MATURITY_PRICE / SATS_PER_BCH
-LONG_PAYOUT_ORACLE_UNITS = UsdEM2Beta(4142.45326)
+LONG_PAYOUT_ORACLE_UNITS_AT_REDEMPTION = UsdEM2Beta(4142.45304)
 
 # Derived total payout sats and units
 # Calculated as SHORT_PAYOUT_SATS_AS_HEDGE + LONG_PAYOUT_SATS
-TOTAL_PAYOUT_SATS_AS_HEDGE = UtxoSats(64283878)
+TOTAL_PAYOUT_SATS_AS_HEDGE = UtxoSats(64283877)
 # Calculated as SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE + LONG_PAYOUT_ORACLE_UNITS
-TOTAL_PAYOUT_ORACLE_UNITS_AS_HEDGE = UsdEM2Beta(14142.4532)
+TOTAL_PAYOUT_ORACLE_UNITS_AS_HEDGE_AT_REDEMPTION = UsdEM2Beta(14142.4532)
 
 # Nominal value at payout
 # Calculated as round_half_up(NOMINAL_VALUE_USDEM2 / float(CLAMPED_PRICE)) * SATS_PER_BCH
 COST_SATS_FOR_NOMINAL_VALUE_AT_REDEMPTION = Sats(45_454_545)
 
+# Create the set of redemption fees
+REDEMPTION_FEES = (
+    FEE_200000_TAKER_TO_MAKER,
+)
+
+# Manually group them by side and unit
+TOTAL_REDEMPTION_FEES_TO_MAKER_SATS = Sats(FEE_200000_TAKER_TO_MAKER.amount_sats)
+TOTAL_REDEMPTION_FEES_TO_TAKER_SATS = Sats(-FEE_200000_TAKER_TO_MAKER.amount_sats)
+
+# Manually calculate total funding and redemption fees in oracle unit terms for checking gains
+TOTAL_FUNDING_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION = UsdEM2Beta(TOTAL_FUNDING_FEES_TO_MAKER_SATS * NAIVE_MATURITY_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+TOTAL_FUNDING_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION = UsdEM2Beta(TOTAL_FUNDING_FEES_TO_TAKER_SATS * NAIVE_MATURITY_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+TOTAL_REDEMPTION_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION = UsdEM2Beta(TOTAL_REDEMPTION_FEES_TO_MAKER_SATS * NAIVE_MATURITY_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+TOTAL_REDEMPTION_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION = UsdEM2Beta(TOTAL_REDEMPTION_FEES_TO_TAKER_SATS * NAIVE_MATURITY_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+
 
 def standard_maturation(
         funding: ContractFunding | None = None,
-        fee_agreements: Sequence[FeeAgreement] = FEES,
+        fee_agreements: Sequence[FeeAgreement] = REDEMPTION_FEES,
 ) -> ContractRedemption:
     funding = funding if funding is not None else standard_contract_funding()
     return funding.redeem(
@@ -485,9 +507,9 @@ def standard_maturation(
 
 
 def standard_liquidation(
-    price_timestamp: ScriptTimestamp = MATURITY_TIMESTAMP,
+    price_timestamp: ScriptTimestamp = LIQUIDATION_TIMESTAMP,
     price_oracleUnits_per_bch: ScriptPriceInOracleUnitsPerBch = LOW_LIQUIDATION_PRICE,
-    fee_agreements: Sequence[FeeAgreement] = FEES,
+    fee_agreements: Sequence[FeeAgreement] = REDEMPTION_FEES,
 ) -> ContractRedemption:
     return standard_contract_funding().redeem(
         price_timestamp=price_timestamp,
@@ -518,25 +540,30 @@ class TestNewRedemptionDerivedValues(unittest.TestCase):
     def test_long_payout_sats_has_correct_value(self):
         self.assertEqual(standard_maturation().long_payout_sats, LONG_PAYOUT_SATS)
 
+    def test_real_duration_seconds_has_correct_value(self):
+        # Note: for maturation, it's actually the price oracle timestamp used for maturation. Here that happens to be set to exactly maturity timestamp.
+        self.assertEqual(standard_maturation().real_duration_seconds, MATURITY_TIMESTAMP - START_TIMESTAMP)
+        self.assertEqual(standard_liquidation().real_duration_seconds, LIQUIDATION_TIMESTAMP - START_TIMESTAMP)
+
     ######################
     # Exact secondary unit values
     ######################
     def test_total_payout_units_has_correct_value(self):
-        self.assertAlmostEqual(standard_maturation().total_payout_oracleUnits, TOTAL_PAYOUT_ORACLE_UNITS_AS_HEDGE, delta=.001)
+        self.assertAlmostEqual(standard_maturation().total_payout_oracleUnits, TOTAL_PAYOUT_ORACLE_UNITS_AS_HEDGE_AT_REDEMPTION, delta=.001)
 
     def test_short_payout_units_has_correct_value(self):
-        self.assertAlmostEqual(standard_maturation().short_payout_oracleUnits, SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE, delta=.001)
+        self.assertAlmostEqual(standard_maturation().short_payout_oracleUnits, SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE_AT_REDEMPTION, delta=.001)
 
     def test_long_payout_units_has_correct_value(self):
-        self.assertAlmostEqual(standard_maturation().long_payout_oracleUnits, LONG_PAYOUT_ORACLE_UNITS, delta=.001)
+        self.assertAlmostEqual(standard_maturation().long_payout_oracleUnits, LONG_PAYOUT_ORACLE_UNITS_AT_REDEMPTION, delta=.001)
 
     ######################
     # Exact secondary gain values
     ######################
     def test_gain_sats(self):
         # Short Maker / Long Taker
-        expected_gain_sats_to_short_maker_at_redemption = SHORT_PAYOUT_SATS_AS_HEDGE - SHORT_INPUT_SATS_AS_HEDGE + TOTAL_FEES_TO_MAKER_SATS
-        expected_gain_sats_to_long_taker_at_redemption = LONG_PAYOUT_SATS - LONG_INPUT_SATS + TOTAL_FEES_TO_TAKER_SATS
+        expected_gain_sats_to_short_maker_at_redemption = SHORT_PAYOUT_SATS_AS_HEDGE - SHORT_INPUT_SATS_AS_HEDGE + TOTAL_FUNDING_FEES_TO_MAKER_SATS + TOTAL_REDEMPTION_FEES_TO_MAKER_SATS
+        expected_gain_sats_to_long_taker_at_redemption = LONG_PAYOUT_SATS - LONG_INPUT_SATS + TOTAL_FUNDING_FEES_TO_TAKER_SATS + TOTAL_REDEMPTION_FEES_TO_TAKER_SATS
         short_maker_contract = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.SHORT)))
 
         # Short Maker
@@ -548,8 +575,8 @@ class TestNewRedemptionDerivedValues(unittest.TestCase):
         self.assertEqual(short_maker_contract.taker_gain_sats, expected_gain_sats_to_long_taker_at_redemption)
 
         # Long Maker / Short Taker
-        expected_gain_sats_to_long_maker_at_redemption = LONG_PAYOUT_SATS - LONG_INPUT_SATS + TOTAL_FEES_TO_MAKER_SATS
-        expected_gain_sats_to_short_taker_at_redemption = SHORT_PAYOUT_SATS_AS_HEDGE - SHORT_INPUT_SATS_AS_HEDGE + TOTAL_FEES_TO_TAKER_SATS
+        expected_gain_sats_to_long_maker_at_redemption = LONG_PAYOUT_SATS - LONG_INPUT_SATS + TOTAL_FUNDING_FEES_TO_MAKER_SATS + TOTAL_REDEMPTION_FEES_TO_MAKER_SATS
+        expected_gain_sats_to_short_taker_at_redemption = SHORT_PAYOUT_SATS_AS_HEDGE - SHORT_INPUT_SATS_AS_HEDGE + TOTAL_FUNDING_FEES_TO_TAKER_SATS + TOTAL_REDEMPTION_FEES_TO_TAKER_SATS
         long_maker_contract = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG)))
 
         # Long Maker
@@ -562,11 +589,12 @@ class TestNewRedemptionDerivedValues(unittest.TestCase):
 
     def test_gain_oracleUnits(self):
         # Short Maker / Long Taker
+        # Note that as the unit of account, we count the SATS value of funding fees, converted to oracleUnits *at the time of redemption*
         expected_gain_oracleUnits_to_short_maker_at_redemption = UsdEM2Beta(
-            SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE - SHORT_INPUT_ORACLE_UNITS_AS_HEDGE + TOTAL_FEES_TO_MAKER_ORACLE_UNITS
+            SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE_AT_REDEMPTION - SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START + TOTAL_FUNDING_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION + TOTAL_REDEMPTION_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION
         )
         expected_gain_oracleUnits_to_long_taker_at_redemption = UsdEM2Beta(
-            LONG_PAYOUT_ORACLE_UNITS - LONG_INPUT_ORACLE_UNITS + TOTAL_FEES_TO_TAKER_ORACLE_UNITS
+            LONG_PAYOUT_ORACLE_UNITS_AT_REDEMPTION - LONG_INPUT_ORACLE_UNITS_AT_START + TOTAL_FUNDING_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION + TOTAL_REDEMPTION_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION
         )
         short_maker_contract = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.SHORT)))
 
@@ -580,10 +608,10 @@ class TestNewRedemptionDerivedValues(unittest.TestCase):
 
         # Long Maker / Short Taker
         expected_gain_oracleUnits_to_long_maker_at_redemption = UsdEM2Beta(
-            LONG_PAYOUT_ORACLE_UNITS - LONG_INPUT_ORACLE_UNITS + TOTAL_FEES_TO_MAKER_ORACLE_UNITS
+            LONG_PAYOUT_ORACLE_UNITS_AT_REDEMPTION - LONG_INPUT_ORACLE_UNITS_AT_START + TOTAL_FUNDING_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION + TOTAL_REDEMPTION_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION
         )
         expected_gain_oracleUnits_to_short_taker_at_redemption = UsdEM2Beta(
-            SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE - SHORT_INPUT_ORACLE_UNITS_AS_HEDGE + TOTAL_FEES_TO_TAKER_ORACLE_UNITS
+            SHORT_PAYOUT_ORACLE_UNITS_AS_HEDGE_AT_REDEMPTION - SHORT_INPUT_ORACLE_UNITS_AS_HEDGE_AT_START + TOTAL_FUNDING_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION + TOTAL_REDEMPTION_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION
         )
         long_maker_contract = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG)))
 
@@ -603,8 +631,8 @@ class TestNewRedemptionDerivedValues(unittest.TestCase):
         short_maker_redemption = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.SHORT)))
         long_maker_redemption = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG)))
 
-        expected_fees_sats_to_maker = TOTAL_FEES_TO_MAKER_SATS
-        expected_fees_sats_to_taker = TOTAL_FEES_TO_TAKER_SATS
+        expected_fees_sats_to_maker = TOTAL_REDEMPTION_FEES_TO_MAKER_SATS
+        expected_fees_sats_to_taker = TOTAL_REDEMPTION_FEES_TO_TAKER_SATS
 
         # Short Maker
         self.assertEqual(short_maker_redemption.fee_sats_to_short, expected_fees_sats_to_maker)
@@ -626,10 +654,9 @@ class TestNewRedemptionDerivedValues(unittest.TestCase):
         # Confirm short maker and long taker
         short_maker_redemption = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.SHORT)))
         long_maker_redemption = standard_maturation(standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG)))
-        oracle_class = short_maker_redemption.base_funding.base_proposal.oracle_unit_cls
 
-        expected_fee_oracleUnits_to_maker_at_redemption = oracle_class(TOTAL_FEES_TO_MAKER_SATS * NAIVE_MATURITY_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
-        expected_fee_oracleUnits_to_taker_at_redemption = oracle_class(TOTAL_FEES_TO_TAKER_SATS * NAIVE_MATURITY_PRICE_USDEM2_PER_BCH / SATS_PER_BCH)
+        expected_fee_oracleUnits_to_maker_at_redemption = TOTAL_REDEMPTION_FEES_TO_MAKER_ORACLE_UNITS_AT_REDEMPTION
+        expected_fee_oracleUnits_to_taker_at_redemption = TOTAL_REDEMPTION_FEES_TO_TAKER_ORACLE_UNITS_AT_REDEMPTION
 
         # Short Maker
         self.assertEqual(short_maker_redemption.fee_oracleUnits_to_short, expected_fee_oracleUnits_to_maker_at_redemption)
@@ -746,11 +773,79 @@ class TestHedgeVsLeveragedShort(unittest.TestCase):
         self.assertEqual(leveraged_short_redemption.cost_sats_for_nominal_value_at_redemption, COST_SATS_FOR_NOMINAL_VALUE_AT_REDEMPTION)
 
 
-class TestContractSide(unittest.TestCase):
-    def test_from_string_works_for_known_api_values(self):
-        self.assertEqual(Side.from_string('short'), Side.SHORT)
-        self.assertEqual(Side.from_string('hedge'), Side.SHORT)
-        self.assertEqual(Side.from_string('long'), Side.LONG)
+class TestIntent(unittest.TestCase):
+    def test_round_trip_from_canonical_intent_to_contract_then_to_bch_input_intent_results_in_identical_contract(self):
+        # start with standard proposals
+        hedge = standard_contract_proposal(short_leverage=ShortLeverage(1.0))
+        short = standard_contract_proposal(short_leverage=ShortLeverage(3.0))
+
+        # create new contracts based on some original values + effective values using the bch input intent
+        hedge_by_short_input = ContractFromInputSatsAndLeverages(
+            start_timestamp=hedge.start_timestamp,
+            maturity_timestamp=hedge.maturity_timestamp,
+            fixed_input_sats=hedge.short_input_sats,
+            fixed_input_side=Side.SHORT,
+            oracle=hedge.oracle_unit_cls,
+            short_leverage=hedge.effective_short_leverage,
+            long_leverage=hedge.effective_long_leverage,
+            start_price_oracleUnits_per_bch=hedge.start_price_oracleUnits_per_bch,
+            maker_side=hedge.maker_side,
+        ).proposal
+        hedge_by_long_input = ContractFromInputSatsAndLeverages(
+            start_timestamp=hedge.start_timestamp,
+            maturity_timestamp=hedge.maturity_timestamp,
+            fixed_input_sats=hedge.long_input_sats,
+            fixed_input_side=Side.LONG,
+            oracle=hedge.oracle_unit_cls,
+            short_leverage=hedge.effective_short_leverage,
+            long_leverage=hedge.effective_long_leverage,
+            start_price_oracleUnits_per_bch=hedge.start_price_oracleUnits_per_bch,
+            maker_side=hedge.maker_side,
+        ).proposal
+        short_by_short_input = ContractFromInputSatsAndLeverages(
+            start_timestamp=short.start_timestamp,
+            maturity_timestamp=short.maturity_timestamp,
+            fixed_input_sats=short.short_input_sats,
+            fixed_input_side=Side.SHORT,
+            oracle=short.oracle_unit_cls,
+            short_leverage=short.effective_short_leverage,
+            long_leverage=short.effective_long_leverage,
+            start_price_oracleUnits_per_bch=short.start_price_oracleUnits_per_bch,
+            maker_side=short.maker_side,
+        ).proposal
+        short_by_long_input = ContractFromInputSatsAndLeverages(
+            start_timestamp=short.start_timestamp,
+            maturity_timestamp=short.maturity_timestamp,
+            fixed_input_sats=short.long_input_sats,
+            fixed_input_side=Side.LONG,
+            oracle=short.oracle_unit_cls,
+            short_leverage=short.effective_short_leverage,
+            long_leverage=short.effective_long_leverage,
+            start_price_oracleUnits_per_bch=short.start_price_oracleUnits_per_bch,
+            maker_side=short.maker_side,
+        ).proposal
+
+        # confirm that the remade contracts exactly match the original contracts
+        # There are possibly unresolvable off-by-one type differences in the final results. Therefore, we can't use simple assertEqual.
+        # Instead we compare the most relevant values and confirm they are very close in % terms.
+        for i, (original, recreated) in enumerate((
+            (hedge, hedge_by_short_input),
+            (hedge, hedge_by_long_input),
+            (short, short_by_short_input),
+            (short, short_by_long_input),
+        )):
+            # Failure message for group:
+            error_msg = f'failed on index {i}'
+
+            # Confirm that some values are exactly equal
+            self.assertEqual(recreated.low_liquidation_price_oracleUnits_per_bch, original.low_liquidation_price_oracleUnits_per_bch, msg=error_msg)
+
+            # Confirm that downstream values are very close. Use BCH since that allows almostEqual to work on standard decimal places
+            self.assertAlmostEqual(recreated.nominal_oracleUnits_x_satsPerBch / SATS_PER_BCH, original.nominal_oracleUnits_x_satsPerBch / SATS_PER_BCH, places=3, msg=error_msg)
+            self.assertAlmostEqual(recreated.total_input_sats.bch, original.total_input_sats.bch, msg=error_msg)
+            self.assertAlmostEqual(recreated.cost_sats_for_nominal_value_at_high_liquidation.bch, original.cost_sats_for_nominal_value_at_high_liquidation.bch, msg=error_msg)
+            self.assertAlmostEqual(recreated.short_input_sats.bch / SATS_PER_BCH, original.short_input_sats.bch / SATS_PER_BCH, msg=error_msg)
+            self.assertAlmostEqual(recreated.long_input_sats.bch / SATS_PER_BCH, original.long_input_sats.bch / SATS_PER_BCH, msg=error_msg)
 
 
 if __name__ == '__main__':
